/*
 *
 * Tachometer pulse converter
 *
 * Juha Niinikoski 06.04.2004
 *
 * 500Hz test generator code 07.04.2004
 *
*/


#include <pic.h>
#include "tacho.h"

/* global variables */

bit feed_dog;				// watch dog feed flag
unsigned char pulse_ctr;		// divider counter



/* included programs */

//#include "timer.c"


/* processor config & id info */

//	__CONFIG(PROTECT75 & INT & WDTEN & PWRTEN & BOREN & LVPDIS);	// WDT enabled

	__CONFIG(UNPROTECT & INT & WDTDIS & PWRTEN & BOREN & LVPDIS);	// WDT disable

	__IDLOC(0103);							// Device ID

const char id[] = {"vJuha Niinikoski 07.04.2004"};	// Visible id string


#define PULSE (0xFFFF - 2000)			// monostabile delay for tacho out  2000us


/* IRQ handler */

void start_pulse(void)				// start tacho output monostabile pulse
	{
	T_OUT = 1;
	TMR1H = PULSE >> 8;
	TMR1L = PULSE;
	TMR1ON = 1;
	}


static void interrupt int_service()
	{
	char dummy;
	if(TMR1IF)				// Tacho out pulse end
		{
		TMR1ON = 0;			// stop timer
		T_OUT = 0;			// end pulse
		TMR1IF = 0;
		}

	if(CMIF)
		{
		if(TEST == 1)			// normal mode, trigger from comparator
			{
			pulse_ctr--;		// count every edge, LSB = 0,5
			if(pulse_ctr == 0)
				{
				start_pulse();	// start tacho pulse
				pulse_ctr = (DIVIDER & 0x7F); // jumper installed = 0-bit
				}
			}

		RA3 = !RA3;  			// comparator test output
		dummy = CMCON;			// reset mismatch
		CMIF = 0;
		}

	if(TMR2IF)				// 1000 triggers/s test generator (500Hz)
		{
		if(TEST == 0)			// if test jumper installed run with 500 Hz
			{
			pulse_ctr--;		// count every edge, LSB = 0,5
			if(pulse_ctr == 0)
				{
				start_pulse();	// start tacho pulse
				pulse_ctr = (DIVIDER & 0x7F);
				}	
			}
		TMR2IF = 0;			// reset IRQ flag
		}
	}


void init_irq(void)
	{
	pulse_ctr = (~DIVIDER & 0x7F);  	// init divider LSB = 0,5

	TMR1IE = 1;				// enable TIMER1, default 1us cycle
	CMIE = 1;				// enable comparator IRQ

	PR2 = 249;				// set TMR2 cycle time 1mS
	T2CKPS0 = 1;				// Prescaler = 4 = 250kHz clock for TMR2
	TMR2ON = 1;				// start TIMER2
	TMR2IE = 1;				// enable TMR2 IRQ

	PEIE = 1;				// enable peripheral IRQs
	GIE = 1;				// enable interrupts
	}



/* helper routines */




/* main alarm logic */

void main(void)
{

	CMCON = CMCON_INIT;			// comparators off
	PORTB = PORTB_INIT;
	PORTA = PORTA_INIT;
	TRISB = TRISB_INIT;			// RB5 - RB7 otputs
	TRISA = TRISA_INIT;

	RBPU = 0;				// enable PORTB pull up resistors for jumper read

	init_irq();				// start timer & comparator IRQ system

	for(;;)
		{
		asm("nop");
		}

}
